import datetime
import os
import sys

import pandas as pd
from paths import in_results, in_look_up, key_col, tabulated, include_habitat, range_run, file_type, look_up_use, \
    overwrite_inter_data, bins, species_prefix

start_time = datetime.datetime.now()
print("Start Time: " + start_time.ctime())

if 0 not in bins:
    bins.append(0)
bins = sorted(bins)
if range_run and file_type != 'Range':
    print('Confirm the inputs are for the ranges files, range_run in the paths.py is set to True, change to False if '
          'you are running to the Critical Habitat files')
    sys.exit()

if not range_run and file_type == 'Range':
    print('Confirm the inputs are for the critical habitat files, range_run in the paths.py is set to False, change '
          'to True if you are running to the range files')
    sys.exit()

if include_habitat and key_col != 'VALUE':
    print("Please confirm your key column (key_col), when including habitat this is typically VALUE and not {0}".format(
        key_col))
    sys.exit()
if range_run and 'Range' not in in_results.split(os.sep):
    print("Please confirm you are working with the Range inputs,your file path for the raw results (in_results) does "
          "not indicate the range files\n{0}".format(in_results))
    sys.exit()

# Confirms the input files match the results

if file_type == 'Range' and 'R' not in os.path.basename(in_look_up).split('_'):
    print('Confirm the look up table matches you data results, you are using the range acres with the critical habitat '
          'results')
    print(in_look_up)
    print(tabulated)
    sys.exit()
elif file_type == 'CriticalHabitat' and 'CH' not in os.path.basename(in_look_up).split('_'):
    print('Confirm the look up table matches you data results, you are using the CH acres with the range results')
    print(in_look_up)
    print(tabulated)
    sys.exit()

if not os.path.exists(os.path.dirname(tabulated)):
    os.mkdir(os.path.dirname(tabulated))
if not os.path.exists(tabulated):
    os.mkdir(tabulated)


def bin_columns(df):
    # NOTES SEE ASSUMPTION ABOUT NOT HAVING ANY "_" in the use name part of final use col headers;
    # ie [region]_[use name]_[interval_value] list of current group of column will not populate correctly
    use_cols = [c for c in df.columns.values.tolist() if c.startswith('VALUE')]
    base_cols = [c for c in df.columns.values.tolist() if not c.startswith('VALUE')]
    if "VALUE" in use_cols:
        use_cols.remove("VALUE")
        base_cols.append("VALUE")

    grouped_use = df[base_cols + use_cols].copy()
    previous_col = []
    # loops over each summary value in the bin list and collapse result for all results found in each summary
    # value. Full impact means each interval in bin list in inclusive of the previous, if this false, each
    # result is independent
    collapse = pd.DataFrame()
    for value in bins:
        binned_col = []
        if bins[0] != 0 and  "VALUE_"+ str(bins[0]) not in df.columns.values.tolist() : # when this happens there is no on-site overlap for that use/species
            print("Check your bins parameters in paths.py. The lowest value distance must be found in the overlap outputs.")
            sys.exit()
        elif value == bins[0]:
                for col in use_cols:
                    get_interval = col.split('_')
                    interval = int(get_interval[(len(get_interval) - 1)])
                    if interval == bins[0]:
                        binned_col.append(col)
        else:
                for col in use_cols:
                    get_interval = col.split('_')
                    interval = int(get_interval[(len(get_interval) - 1)])
                    if interval == 0:
                        continue
                    else:
                        if interval <= value and col not in previous_col:
                            binned_col.append(col)
        for p in binned_col:
                if p in previous_col:
                    binned_col.remove(p)
                previous_col.append(p)
        #print (binned_col, value)  # uncomment if you want to see which columns were included for the bin value
        # combines result into the working output df then saves files
        bin_group = grouped_use[ base_cols + binned_col].copy()
        bin_group.loc[:, binned_col] = bin_group.loc[:, binned_col].apply(pd.to_numeric, errors='coerce')
        col_header = ("VALUE") + '_' + str(value)
        # print (col_header, value)
        bin_group[col_header] = bin_group[binned_col].sum(axis=1)
        binned_result = bin_group[base_cols + [col_header]].copy()
        if len(collapse) == 0:
            collapse = binned_result.copy()
        else:
            common_col = [v for v in binned_result.columns.values.tolist() if v in collapse.columns.values.tolist()]
            collapse = pd.merge( collapse,binned_result, how='outer', on=common_col)

    return collapse


list_results_directory = os.listdir(in_results)
list_lookup = os.listdir(in_look_up)
chem_use = pd.read_csv(look_up_use,encoding ='ISO-8859-1')
# finds column with the names of the raster uses this function because sometimes when saving a csv from an excel the
# first column will have extra characters
full_name =[v for v in chem_use.columns.values.tolist() if v.endswith('FullName')]
include_udls = list(set(chem_use[full_name[0]].values.tolist()))
include_udls = [v +"_euc" for v in include_udls]

for folder in list_results_directory:
    if folder not in include_udls:
        continue
    out_path = tabulated + os.sep + folder
    if not os.path.exists(out_path):
        os.mkdir(out_path)
    csv_list = os.listdir(in_results + os.sep + folder)
    print ("Working on use:{0}".format(folder))

    for csv in csv_list:
        in_csv = csv
        csv = csv.replace("__", "_")
        update_table = False
        if overwrite_inter_data:
            update_table = True
        if not os.path.exists(out_path + os.sep + csv) or not os.path.exists(out_path + os.sep + csv.replace("__", "_")):
            update_table = True

        if update_table:
            results_df = pd.read_csv(in_results + os.sep + folder + os.sep + in_csv, low_memory=False,encoding ='ISO-8859-1')
            check_species_prefix = csv.split("_")[0] + "_" + csv.split("_")[1]
 
            if check_species_prefix not in species_prefix:
                continue
            print ("    Working on table:{0}".format(csv))
    
            if include_habitat:
                lookup_csv = [t for t in list_lookup if t.startswith(csv.split("_")[0] + "_" + csv.split("_")[1])]
            else:
                # print (csv.split("_")[0].upper() + "_" + csv.split("_")[1].capitalize())
                lookup_csv = [t for t in list_lookup if
                              t.startswith(csv.split("_")[0].upper() + "_" + csv.split("_")[1].capitalize())]

            lookup_df = pd.read_csv(in_look_up + os.sep + lookup_csv[0], low_memory=False,encoding ='ISO-8859-1')
            if 'VALUE' in results_df.columns.values.tolist() and key_col in lookup_df.columns.values.tolist():
                merged_df = pd.merge(results_df, lookup_df, how='left', left_on='VALUE', right_on=key_col)
                

            else:
                print(
                    'Confirm the inputs for the Raw Results match your look-up tables for the critical habitat or '
                    'ranges')
                sys.exit()
            [merged_df.drop(cols, axis=1, inplace=True) for cols in merged_df.columns.values.tolist() if
             cols.startswith('Un')]
            binned_drift = bin_columns(merged_df)
            binned_drift.to_csv(out_path + os.sep + csv,encoding ='ISO-8859-1')

end = datetime.datetime.now()
print("End Time: " + end.ctime())
elapsed = end - start_time
print("Elapsed  Time: " + str(elapsed))
