import pandas as pd
import os
import datetime
import sys

from paths import tabulated, grouping_col,tabulated_merge, list_pol_ids, look_up_use, regions, overwrite_inter_data,list_geoid_ids,exclusion_state,exclusion_cnty, bins


start_time = datetime.datetime.now()
print ("Start Time: " + start_time.ctime())

# Functions


def create_directory(dbf_dir):  # creates a directory
    if not os.path.exists(dbf_dir):
        os.mkdir(dbf_dir)

if 0 not in bins:
    bins.append(0)
bins = sorted(bins)
use_lookup = pd.read_csv(look_up_use, low_memory=False, encoding = "ISO-8859-1")
list_results_directory = os.listdir(tabulated)  # gets a list of folders in the tabulated director
# Loops over each folder and gets a list of csvs (by species group) and summarizes them to either the state or county
# based on the user input in the path script. This is typically the county
# output a single merged table that includes all species for a give use

# finds column with the names of the raster uses this function because sometimes when saving a csv from an excel the
# first column will have extra characters
full_name =[v for v in use_lookup.columns.values.tolist() if v.endswith('FullName')]
all_uses_gis = use_lookup[full_name[0]].values.tolist()
all_uses_gis = [v +'_euc' for v in all_uses_gis]
print ("Uses that are included for your chemical {0}".format(all_uses_gis))
for folder in list_results_directory:
    region = folder.split("_")[0]
    if region not in regions:
        continue
    if folder not in all_uses_gis:
        continue
    use = use_lookup.loc[use_lookup['FullName']==(folder.replace("_euc","")),'FinalUseHeader'].iloc[0]
    print ('\nWorking on use {0} in folder {1}'.format(use,folder))
    csv_list = os.listdir(tabulated + os.sep+ folder)  # list of results in the folder
    csv_list = [v for v in csv_list if v.endswith('.csv')]  # limited to .csv files
    merged_df = pd.DataFrame()  # empty dataframe to store the merged use result
    update_table = False
    if overwrite_inter_data:
        update_table = True
    if (len(grouping_col) == 3 and not os.path.exists(tabulated_merge + os.sep + folder +'_Cnty.csv')) or(len(grouping_col) < 3 and not os.path.exists(tabulated_merge + os.sep + folder +'_State.csv')):
        update_table = True
    if update_table:
        for csv in csv_list:
            print ("  Loading species group {0}".format(csv))
            results_df = pd.DataFrame()
            for chunk in pd.read_csv(tabulated + os.sep + folder + os.sep + csv, low_memory=False, chunksize=100000,encoding ='ISO-8859-1'):
                results_df = pd.concat([results_df, chunk], ignore_index=True)
            # results_df = pd.read_csv(tabulated + os.sep + folder + os.sep + csv, low_memory=False,encoding ='ISO-8859-1')
            results_df['EntityID'] = results_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
            if 'GEOID' in results_df.columns.values.tolist():
                results_df['GEOID'] = results_df['GEOID'].map(lambda x: str(x).split(".")[0] if len(str(x).split(".")[0]) == 5 else '0' + str(x).split(".")[0]).astype(str)
                if len(list_geoid_ids)>0:
                    if exclusion_cnty:
                        results_df = results_df.loc[~results_df['GEOID'].isin(list_geoid_ids)].copy()
                    else:
                        results_df = results_df.loc[results_df['GEOID'].isin(list_geoid_ids)].copy()
                
            if len(list_pol_ids)>0:
                if exclusion_state:
                    results_df = results_df.loc[~results_df['STUSPS'].isin(list_pol_ids)].copy()
                else:
                    results_df = results_df.loc[results_df['STUSPS'].isin(list_pol_ids)].copy()
            # get a list of columns that state with VALUE - these are the overlap results. VALUE_[distance] is the
            # overlap
            # results and VALUE is the zone key for the results
            val_col = [v for v in results_df.columns.values.tolist() if v.startswith('VALUE')]
           
            if 'VALUE' in val_col:  # zone key for overlap run in the results file, this is no longer needed
                val_col.remove('VALUE')
            # copies the grouping columns (the political boundaries IDs and Entity ID with the overlap results in
            # the value columns to a dataframe
            df_pol = results_df[grouping_col + val_col].copy()
            col_order = grouping_col + val_col  # column order for re-index of columns
            # execute the group-by and the re-index of the columns
            df_pol = df_pol.groupby(grouping_col)[val_col].sum().reset_index()
            df_pol = df_pol.reindex(columns=col_order)
            if bins[0] != 0 and  "VALUE_"+ str(bins[0]) not in df_pol.columns.values.tolist() : # when this happens there is no on-site overlap for that use/species
                print("Check your bins parameters in paths.py. The lowest value distance must be found in the overlap outputs.")
                sys.exit()
            elif "VALUE_0" not in df_pol.columns.values.tolist()and bins[0] == 0:
                df_pol["VALUE_0"] =0
                
            # appends the results of the group-by to the working merged df for the use
            if len(merged_df) == 0:
                merged_df = df_pol.copy()
            else:
                values_cols = [v for v in df_pol.columns.values.tolist() if v.startswith('VALUE')]
                # list of columns that will be found in both the working merge table and the group-by. This is used as
                # the key to combine date when merging
                merge_cols = grouping_col + values_cols
                common_cols = []
                for col in merge_cols:
                    if col in merged_df.columns.values.tolist():
                        common_cols.append(col)

                merged_df = pd.merge(merged_df, df_pol, left_on= common_cols, right_on= common_cols, how='outer')
                merged_df.fillna(0,inplace=True)
            

        # Saves the resulting table that includes all the species results in one table to appropriate folder
        if len(grouping_col) == 3:
            create_directory(tabulated_merge)
            merged_df.to_csv(tabulated_merge + os.sep + folder +'_Cnty.csv',encoding ='ISO-8859-1')


        else:
            create_directory(tabulated_merge)
            merged_df.to_csv(tabulated_merge + os.sep + folder +'_State.csv',encoding ='ISO-8859-1')

    else:
        print ("Already completed tables for {0}. If you wish to update change the overwrite_inter_data to True".format(folder))


end = datetime.datetime.now()
print ("End Time: " + end.ctime())
elapsed = end - start_time
print ("Elapsed  Time: " + str(elapsed))
