import datetime
import os
import sys

import pandas as pd


from paths import tabulated_merge, in_acres_table, grouping_col, file_type, master_list, col_include_output, \
    look_up_use, chemical_name, per_overlap_path, final_path, bins, only_include_bins, overwrite_inter_data, python_version_three
if python_version_three:
    # ignore performance warnings
    from warnings import simplefilter
    simplefilter(action="ignore", category=pd.errors.PerformanceWarning)

# Title- Generate overlap tables from zone species rasters to use layers results;

# ASSUMPTIONS
# Species group is found in index position 1 of all input result tables when split by '_'
# All raster are 30-meter cells - note previously VI and CNMI has some use with a different cell size

# distance 0 must in the bin list, adds vaile if missing from user inputs and sorts from low to high
if 0 not in bins:
    bins.append(0)
bins = sorted(bins)

# ###Functions


def create_directory(dbf_dir):
    if not os.path.exists(dbf_dir):
        os.mkdir(dbf_dir)


def calculation(type_fc, in_sum_df, c_region, percent_type, id_cols):
    # ASSUMES ONLY NUMERIC COLS ARE USE COLS AND ACRES COLS
    # Calculates percent overlap
    use_cols = [f for f in in_sum_df.columns.values.tolist() if len(f.split("_")) == 2]

    for col_value in id_cols:
        if col_value in use_cols:
            use_cols.remove(col_value)
            in_sum_df[col_value] = in_sum_df[col_value].map(lambda x: x).astype(str)

    if percent_type == 'full range':
        acres_col = 'TotalAcresOnLand'
        if acres_col not in use_cols:
            use_cols.append(acres_col)
        in_sum_df[use_cols] = in_sum_df[use_cols].apply(pd.to_numeric,errors='coerce')
        in_sum_df = in_sum_df[in_sum_df[acres_col].notnull()]
    elif percent_type == 'NL48 range':
        acres_col = 'TotalAcresNL48'
        if acres_col not in use_cols:
            use_cols.append(acres_col)
        in_sum_df[use_cols] = in_sum_df[use_cols].apply(pd.to_numeric,errors='coerce')
        in_sum_df = in_sum_df[in_sum_df[acres_col].notnull()]
    elif percent_type == 'regional range':
        acres_col = 'Acres_' + str(c_region)
        if acres_col not in use_cols:
            use_cols.append(acres_col)
        in_sum_df[use_cols] = in_sum_df[use_cols].apply(pd.to_numeric,errors='coerce')
        in_sum_df = in_sum_df[in_sum_df[acres_col].notnull()]
    use_cols.remove(acres_col)
    if type_fc == "Raster":
        overlap = in_sum_df.copy()

        # convert msq to acres for the use
        overlap[use_cols] *= 0.000247

        # generate percent overlap by taking acres of use divided by total acres of the species range
        overlap[use_cols] = overlap[use_cols].div(overlap[acres_col], axis=0)
        overlap[use_cols] *= 100

        # Drop excess acre col- both regional and full range are included on input df; user defined parameter
        # percent_type determines which one is used in overlap calculation
        if 'TotalAcresNL48' in overlap.columns.values.tolist():
            overlap.drop('TotalAcresNL48', axis=1, inplace=True)
        if 'Acres_CONUS' in overlap.columns.values.tolist():
            overlap.drop('Acres_CONUS', axis=1, inplace=True)
        if 'TotalAcresOnLand' in overlap.columns.values.tolist():
            overlap.drop('TotalAcresOnLand', axis=1, inplace=True)

        return overlap, acres_col

    else:
        print ("ERROR ERROR: could not locate acre information. Please confirm inputs")
        sys.exit()


def check_for_empty_row(df_in):
    df = df_in.copy()
    overlap_cols = [p for p in df.columns.values.tolist() if p.startswith('CONUS') or p.startswith('NL48')]
    df[overlap_cols] = df[overlap_cols].apply(pd.to_numeric, errors='coerce')
    w_df = df.copy()
    w_df['SUM'] = w_df[overlap_cols].sum(axis=1)
    w_df = w_df.loc[w_df['SUM'] > 0]
    w_df.drop('SUM', axis=1, inplace=True)
    return w_df, overlap_cols


def overlap_table (list_csv, out_path,in_folder,runtype, today_date, table_type):
    if runtype =='pctn':
        # change the path to be a second folder with suffix of PCTn for the tables associated with PCTns
        out_path = out_path + "_PCTns"
    else:
        # remove any grouped NL48 percent overlap from a previous run otherwise it will be added into the final table
        if os.path.exists(out_path):
            list_old_csv = os.listdir(out_path)
            if len(list_old_csv) > 0:
                [os.remove(per_overlap_path + os.sep + v) for v in list_old_csv if 'NL48' in v.split("_")]

    use_list = []  # placeholder list to track uses
    for csv in list_csv:  # loops over each output table to generate the percent overlap table
        # STEP 1: Loads the inputs need to complete the calculation
        # extract use name and region from csv name
        if 'Cnty.csv' in csv.split("_"):
            csv_suffix_result = '_euc_Cnty.csv'
        elif 'State.csv' in csv.split("_"):
            csv_suffix_result = '_euc_State.csv'
        elif csv.endswith('PCTn.csv'):
            if csv.endswith("_Uniform_State_AVG_PCTn.csv"):
                csv_suffix_result = "_euc_Uniform_State_AVG_PCTn.csv"
            elif csv.endswith("_Upper_State_AVG_PCTn.csv"):
                csv_suffix_result = "_euc_Upper_State_AVG_PCTn.csv"
            elif csv.endswith("_Uniform_State_MAX_PCTn.csv"):
                csv_suffix_result = "_euc_Uniform_State_MAX_PCTn.csv"
            elif csv.endswith("_Upper_State_MAX_PCTn.csv"):
                csv_suffix_result = "_euc_Upper_State_MAX_PCTn.csv"
        else:
            csv_suffix_result = '_euc.csv'
        use_nm = csv.replace(csv_suffix_result, '')  # extract the use name used to id the folder using the filename
        if use_nm not in use_list:
            use_list.append(use_nm)

        if use_nm not in use_lookup['FullName'].values.tolist():  # skips any uses that are pertinent to the chemical
            continue
        region = use_nm.split("_")[0]
        # loaded information from the acres tables
        acres_for_calc = acres_df[['EntityID', 'Acres_CONUS', 'TotalAcresNL48', 'TotalAcresOnLand']].copy()
        acres_for_calc['EntityID'] = acres_for_calc.loc[:, 'EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)

        # load use information for chemical from use lookup for the specific use
        type_use = use_lookup.loc[use_lookup['FullName'] == use_nm, 'Type'].iloc[0]
        final_col_header = use_lookup.loc[use_lookup['FullName'] == use_nm, 'Chem Table FinalColHeader'].iloc[0]
        print (
            "\nYou are working on results from {0}, and the final header will be {1}".format(use_nm, final_col_header))

        # loads input overlap tables and set the entity ID as str
        print ("Loaded table {0}".format(csv))
        use_array = pd.read_csv(in_folder + os.sep + csv, low_memory=False, encoding='utf-8')
        if len(use_array) == 0:  # species/use  has not overlap
            continue
        # sets entity ID as a string
        use_array['EntityID'] = use_array['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
        if only_include_bins:  # trims out distance columns that are greater than max distance set by user
            use_cols = [v for v in use_array.columns.values.tolist() if v.startswith('VALUE')]
            sp_cols = [v for v in use_array.columns.values.tolist() if not v.startswith('VALUE')]
            keep_distance = [v for v in use_cols if int(str(v.split("_")[1])) <= max(bins)]
            # print (sp_cols+ keep_distance)
            use_array = use_array[sp_cols + keep_distance].copy()

        # STEP 2: Run percent overlap for both the full range and the regional; set up acre table to include only
        # species found on the current use table (use_array)

        # create folder for percent overlap files and sent the out paths and file names
        create_directory(out_path)
        out_use_overlap = out_path + os.sep + 'PercentOverlap_' + final_col_header + "_" + date + '.csv'
        if csv.endswith('PCTn.csv'):
            # print (csv.split())
            # used for final column headers
            if csv.split("_")[1] == "CDL":
                pct = (csv.split("_")[5]+"_"+csv.split("_")[6]+"_"+csv.split("_")[7]).replace('.csv','')
            else:
                pct = (csv.split("_")[3]+"_"+csv.split("_")[4]+"_"+csv.split("_")[5]).replace('.csv','')
            # print (pct)
            final_col_header = final_col_header +"_"+ pct
            #changes output location for PCTn tables and saves to a different folder
            out_use_overlap = out_path + os.sep + 'PercentOverlap_' + final_col_header + "_" + date + '.csv'

        # skips use if the user has chosen not to overwrite existing tables
        if os.path.exists(out_use_overlap) and region == 'CONUS' and not overwrite_inter_data:
            print ('Already created {0}'.format(out_use_overlap))
            continue
        else:
            # merges use table with the acres information
            sum_df_acres = pd.merge(use_array, acres_for_calc, on='EntityID', how='left')
            print ('Generating percent overlap...:{0}, {1}'.format(csv, final_col_header))
            # runs percent overlap calculation
            if final_col_header.startswith('CONUS'):
                ovrlp_type = 'regional range'
            else:
                ovrlp_type = 'NL48 range'
            # in the calculation formula the use area is convert from msq to acres to align with the area unit for the
            # species acres tables
            percent_overlap_df_region, acre_col = calculation(type_use, sum_df_acres, region, ovrlp_type, grouping_col)
            # update the columns headers to use the header name from the use lookup table
            out_col = []
            for col in percent_overlap_df_region.columns.values.tolist():
                if col.startswith("VALUE"):
                    i = col.replace("VALUE", final_col_header)
                    out_col.append(i)
                else:
                    out_col.append(col)
            percent_overlap_df_region.columns = out_col
            [percent_overlap_df_region.drop(m, axis=1, inplace=True) for m in
             percent_overlap_df_region.columns.values.tolist() if m.startswith('Unnamed')]

        if os.path.exists(out_use_overlap) and final_col_header.startswith("NL48"):

            print ('Add NL48 use/region: {0}'.format(csv))
            # sum across nl48 regions
            nl48_other_region = pd.read_csv(out_use_overlap, encoding='utf-8')
            nl48_other_region['EntityID'] = nl48_other_region['EntityID'].map(lambda x: str(x).split('.')[0]).astype(
                str)
            nl48_other_region.drop([v for v in nl48_other_region.columns.values.tolist() if v.startswith('Unnamed')],
                                   axis=1, inplace=True)
            nl48_other_region['EntityID'] = nl48_other_region['EntityID'].map(lambda x: str(x).split('.')[0]).astype(
                str)
            # print (out_use_overlap)
            percent_overlap_df_region = pd.concat([percent_overlap_df_region, nl48_other_region], axis=0)
            if 'GEOID' in grouping_col and not table_type:
                percent_overlap_df_region = percent_overlap_df_region.groupby(['EntityID', 'STUSPS', 'GEOID'],
                                                                              as_index=False).sum()
            elif 'STUSPS' in grouping_col:
                percent_overlap_df_region = percent_overlap_df_region.groupby(['EntityID', 'STUSPS'],
                                                                              as_index=False).sum()
            else:
                percent_overlap_df_region = percent_overlap_df_region.groupby(['EntityID'], as_index=False).sum()

        # confirms columns are in numerical order, table will become out of order when some regions that don't have
        # overlap at every distance esp VI
        dict_col = {}
        base_col = []
        for col in percent_overlap_df_region.columns.values.tolist():
            if col.startswith('CONUS') or col.startswith('NL48'):
                col_splt = col.split("_")
                dict_col[col] = int(col.split("_")[len(col_splt)-1])
            else:
                base_col.append(col)
        sort_orders = sorted(dict_col.items(), key=lambda x: x[1], reverse=False)
        final_order = {}
        for i in sort_orders:
            final_order[i[0]] = i[1]
        first_tuple_elements = [a_tuple[0] for a_tuple in sort_orders]
        percent_overlap_df_region = percent_overlap_df_region.reindex(columns=base_col + first_tuple_elements)
        # saves table
        percent_overlap_df_region.to_csv(out_use_overlap, encoding='utf-8', index=False)
        print ("Exported table {0}".format(out_use_overlap))
    return use_list


def merge_table(list_csv, perovrlp_path, out_table_cnty, out_table_state, out_table_species,add_pct):
    # merges all percent overlap tables into one table by looping over each csv
    out_df = pd.DataFrame()

    for t in list_csv:
        print('  Working on use...{0}'.format(t))
        current_csv = perovrlp_path + os.sep + t
        # load the current percent overlap table and sets the entity ID to a string, drops index colds
        in_df = pd.read_csv(current_csv, dtype=object, low_memory=False, encoding='utf-8')
        in_df['EntityID'] = in_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
        [in_df.drop(m, axis=1, inplace=True) for m in in_df.columns.values.tolist() if m.startswith('Unnamed')]
        in_df.fillna(0, inplace=True)

        if 'GEOID' in in_df.columns.values.tolist() and not add_pct:
            in_df['GEOID'] = in_df['GEOID'].map(lambda x: str(x) if len(str(x)) == 5 else '0' + str(x)).astype(str)
        # capture NL48 tables, but will not refine by state or county
        if 'NL48' in t.split("_"):
            if len(grouping_col) == 2:
                if 'GEOID' not in in_df.columns.values.tolist():
                    in_df['GEOID'] = pd.Series(["99999" for x in range(len(in_df.index))])
            if 'STUSPS' in grouping_col:
                if 'STUSPS' not in in_df.columns.values.tolist():
                    in_df['STUSPS'] = pd.Series(["NL48" for x in range(len(in_df.index))])
        if len(out_df)<1:
            out_df= in_df.copy()
        else:
            [in_df.drop(m, axis=1, inplace=True) for m in in_df.columns.values.tolist() if m.startswith('Drift Ratio')]
            [out_df.drop(m, axis=1, inplace=True) for m in out_df.columns.values.tolist() if m.startswith('Drift Ratio')]
            common_col = [v for v in out_df.columns.values.tolist() if v in in_df.columns.values.tolist()]
            out_df = pd.merge(out_df, in_df, how='outer', on=common_col)

    out_df.fillna(0, inplace=True)
    out_df, overlap_cols = check_for_empty_row(out_df)
    out_df[overlap_cols] = out_df[overlap_cols].apply(pd.to_numeric, errors='coerce')
    # If cnty identifiers are in the overlap tables generates the cnty table then move to state table
    # does not append pct to cnty table 
    if 'GEOID' in grouping_col and not add_pct:
        if 'GEOID' in out_df.columns.values.tolist():
            print ("    Grouping for county/all uses table")
            cnt_time = datetime.datetime.now()
            out_df['GEOID'] = out_df['GEOID'].map(lambda x: str(x) if len(str(x)) == 5 else '0' + str(x)).astype(str)
            working_df = out_df.copy()
            df_cnty = working_df.groupby(['EntityID', 'STUSPS','GEOID'], as_index=False)[overlap_cols].sum().reset_index().copy()
            if os.path.exists(out_table_cnty):
                saved_df = pd.read_csv(out_table_cnty)
                saved_df['EntityID'] = saved_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
                [saved_df.drop(m, axis=1, inplace=True) for m in saved_df.columns.values.tolist() if m.startswith('Unnamed')]
                saved_df['GEOID'] = saved_df['GEOID'].map(lambda x: str(x) if len(str(x)) == 5 else '0' + str(x)).astype(str)
                common_col = [v for v in saved_df.columns.values.tolist() if v in df_cnty.columns.values.tolist()]
                df_cnty = pd.merge(saved_df, df_cnty, how='left', on=common_col)
            # saves the new parent table that includes the use
            df_cnty.to_csv(out_table_cnty, index=False, encoding='utf-8')
            print ("    Grouped and merge cnty in {0}".format(datetime.datetime.now() - cnt_time))

    # If state identifiers are in the overlap tables generates the state table then move to species table
    if 'STUSPS' in grouping_col:
        print ("    Grouping for state/all uses table")
        state_time = datetime.datetime.now()
        if 'GEOID' in out_df.columns.values.tolist():
            # drops GEOID if it is present do it can be summed by species state
            df_state = out_df.drop('GEOID', axis=1)
        else:
            df_state=out_df.copy()
        df_state_working = df_state.copy()
        state_df_sum = df_state_working.groupby(['EntityID', 'STUSPS'], as_index=False)[overlap_cols].sum().reset_index().copy()
        state_df_sum ['EntityID'] = state_df_sum ['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
        if os.path.exists(out_table_state):
            saved_df = pd.read_csv(out_table_state)
            saved_df['EntityID'] = saved_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
            [saved_df.drop(m, axis=1, inplace=True) for m in saved_df.columns.values.tolist() if m.startswith('Unnamed')]
            state_df_sum = pd.merge(saved_df, state_df_sum, how='left', on=['EntityID','STUSPS'])
        # saves the new parent table that includes the use
        state_df_sum.to_csv(out_table_state, index=False, encoding='utf-8')
        print ("    Grouped and merge state in {0}".format(datetime.datetime.now() - state_time))
    if 'EntityID' in out_df.columns.values.tolist():
        print ("    Grouping for species/all uses  table")
        sp_time = datetime.datetime.now()
        if 'GEOID' in out_df.columns.values.tolist():  # drops county id if present to sum by species
            df_state = out_df.drop('GEOID', axis=1)
        else:
            df_state = out_df.copy()
        if 'STUSPS'in df_state.columns.values.tolist():  # drop state id if present to sum by species
            df_species= df_state.drop('STUSPS', axis=1)
        else:
            df_species = df_state.copy()
        df_species_sum = df_species.groupby('EntityID', as_index=False)[overlap_cols].sum().reset_index().copy()
        df_species_sum['EntityID'] = df_species_sum['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
        if os.path.exists(out_table_species):
            saved_df = pd.read_csv(out_table_species)
            saved_df['EntityID'] = saved_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
            [saved_df.drop(m, axis=1, inplace=True) for m in saved_df.columns.values.tolist() if m.startswith('Unnamed')]
            df_species_sum= pd.merge(saved_df, df_species_sum, how='left', on=['EntityID'])
        # saves the new parent table that includes the use
        df_species_sum.to_csv(out_table_species, index=False, encoding='utf-8')
        print ("    Grouped and merge species in {0}".format(datetime.datetime.now() - sp_time))



def final_parent_table(out_table, grping_col):
    # confirms all columns and data in the parent tables are present and summarized correctly
    final_df = pd.read_csv(out_table, low_memory=False, encoding='utf-8')
    final_df['EntityID'] = final_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
    final_df, overlap_cols = check_for_empty_row(final_df)
    final_df[overlap_cols] = final_df[overlap_cols].apply(pd.to_numeric, errors='coerce')
    w_df = final_df.copy()
    # sums to the corresponding grouping columns
    if os.path.exists(out_table) and 'GEOID' in grping_col:
        out_df = w_df.groupby(grping_col, as_index=False)[overlap_cols].sum().reset_index().copy()
    elif os.path.exists(out_table) and 'STUSPS' in grping_col:
        out_df = w_df.groupby( grping_col, as_index=False)[overlap_cols].sum().reset_index().copy()
    else:
        out_df = w_df.groupby(grping_col, as_index=False)[overlap_cols].sum().reset_index().copy()
    # appends the columns from the master species list

    common_col = [v for v in base_sp_df.columns.values.tolist() if v in out_df.columns.values.tolist()]
    out_df = pd.merge(base_sp_df, out_df, how='left', on=common_col)
    out_df.fillna(0, inplace=True)  # fills blanks
    [out_df.drop(m, axis=1, inplace=True) for m in out_df.columns.values.tolist() if m.startswith('index')]
    out_df.to_csv(out_table, encoding='utf-8')
    print("Exported final parent table{0}".format(out_table))


start_time = datetime.datetime.now()
print ("Start Time: " + start_time.ctime())

# sets the folder with where the summary files will be found

in_path = tabulated_merge

# confirm summary folder exists
if not os.path.dirname(in_path):
    print('Cannot find the merged folder in the output directory; confirm input values on the paths.py')
    sys.exit()

# Confirms the input files match the results
if file_type == 'Range' and not os.path.basename(in_acres_table).startswith('R'):
    print('Confirm the acres table matches you data results, you are using the range acres with the critical habitat '
          'results')
    print(in_acres_table)
    print (in_path)
    sys.exit()
elif file_type == 'CriticalHabitat' and not os.path.basename(in_acres_table).startswith('CH'):
    print('Confirm the acres table matches you data results, you are using the CH acres with the range results')
    print(in_acres_table)
    print (in_path)
    sys.exit()

# date

today = datetime.datetime.today()
date = today.strftime('%Y%m%d')
run_pctn = False  # default value assumes PCTns aren't available

# load information from input tables and sets the EntityID col as a string
species_df = pd.read_csv(master_list, dtype=object, low_memory=False, encoding = 'ISO-8859-1')
[species_df.drop(m, axis=1, inplace=True) for m in species_df.columns.values.tolist() if m.startswith('Unnamed')]
species_df['EntityID'] = species_df['EntityID'].map(lambda k: str(k).split('.')[0]).astype(str)
base_sp_df = species_df.loc[:, col_include_output]

# loads species acres needed to created percent overlap
acres_df = pd.read_csv(in_acres_table, low_memory=False, encoding = 'ISO-8859-1')
acres_df['EntityID'] = acres_df['EntityID'].map(lambda k: str(k).split('.')[0]).astype(str)

# load chemical use table
use_lookup = pd.read_csv(look_up_use, low_memory=False, encoding = 'utf-8')

# Get a list of use tables in area found in the output folder(s)
all_csv_files = [v for v in os.listdir(in_path) if  v.endswith('.csv')]
# converts units from meters squared to acres then calculated the percent overlap tables
std_use = overlap_table(all_csv_files ,per_overlap_path, in_path,'std',date, run_pctn)

# if you have generated pct tables for a use, gets a list of those tables and then concert from meters squared to acres
# and calculates overlap

if os.path.exists(in_path +"_PCTns"):
    pctn_csv = [v for v in os.listdir(in_path +"_PCTns") if v.endswith('.csv')]
    run_pctn = True
if run_pctn:
    pct_uses = overlap_table(pctn_csv, per_overlap_path,in_path +"_PCTns",'pctn',date, run_pctn)
    
print ("\nFinished Percent Overlap tables in {0}".format(datetime.datetime.now() - start_time))

# STEP 3: Merge percent overlap tables into one table for the chemical/ create "parent table" for chemical that includes
# all defined uses
# deletes overlap tables from today, and removes an overlap table list if it was based on a previous date/run

# get list(s) of tables to be included in the parent tables. List will only include overlap table with the current date
# list for standard uses
list_percent_csv = os.listdir(per_overlap_path)
list_percent_csv = [csv for csv in list_percent_csv if csv.endswith('.csv')]
# confirms the date on file, old files excluded
[list_percent_csv.remove(v) for v in list_percent_csv if (v.split("_")[3]).replace('.csv','')!= date]

# list for PCTns (if the chemical has them_
if os.path.exists(per_overlap_path +"_PCTns"):
    pctn_percent = [v for v in os.listdir(per_overlap_path +"_PCTns") if v.endswith('.csv')]
    for v in pctn_percent:
        # confirms the date on file, old files excluded
        if v.split("_")[6].replace('.csv', '') != date:
            print ("Exluded old table:".format(v))
            pctn_percent .remove(v)
    run_pctn = True

# path for the Parent tables
final_path = final_path +os.sep+"ParentTables"
# create folder if it doesn't exist
if not os.path.exists(final_path):
    os.mkdir(final_path)

# file names for the parent tables
cnty_table = final_path + os.sep + file_type + "_" + chemical_name + '_AllUses_' + date + '_Cnty.csv'
state_table =final_path + os.sep + file_type + "_" + chemical_name + '_AllUses_' + date + '_State.csv'
species_table = final_path + os.sep + file_type + "_" + chemical_name + '_AllUses_' + date + '.csv'

# removes any other parent table that were generated with the current date
for final_table in [cnty_table,state_table,species_table]:
    if os.path.exists(final_table):
        os.remove(final_table)

print ('\nMerging overlap table for all uses...')
# merges the standard uses for the chemical
pct_table=False
merge_table(list_percent_csv, per_overlap_path, cnty_table, state_table, species_table, pct_table)
# if you have PCT table adds in the PCT overlap columns
if run_pctn:
    print("\nAdding PCTn overlaps")
    pct_table = True
    per_overlap_path_pctn = per_overlap_path +"_PCTns"
    merge_table(pctn_percent,per_overlap_path_pctn,cnty_table, state_table, species_table, pct_table)
#
# cleans up the final tables after merging the uses together
if os.path.exists(cnty_table):
    final_parent_table(cnty_table,['EntityID','STUSPS','GEOID'])

if os.path.exists(state_table):
    final_parent_table(state_table,['EntityID','STUSPS'])

if os.path.exists(species_table):
    final_parent_table(species_table,['EntityID'])

end = datetime.datetime.now()
print ("End Time: " + end.ctime())
elapsed = end - start_time
print ("Elapsed  Time: " + str(elapsed))
