import pandas as pd
import os
import datetime
import sys
from paths import tabulated, look_up_use, python_version_three, on_off_excel,sheet_name_on_off, final_path, range_run

# creates the on/off tables for the chemicals
start_time = datetime.datetime.now()
print ("Start Time: " + start_time.ctime())


if not range_run:
    print ("The on/off field adjustment is not done for Critical Habitat, your range_run parameter is set to False")
    sys.exit()

final_path = final_path +os.sep+"ParentTables"

def apply_on_off(use_look_df, on_off, in_df, col_header_on_off):
    # applies the on/off site adjustment to the direct overlap based on use input
    onoff_cols = []
    # print (use_look_df[col_header_on_off])
    # print (use_look_df.columns.values.tolist())
    if col_header_on_off not in use_look_df.columns.values.tolist():
        print ("   There are no on/off uses for this chemical in on/off categories: {0}".format(col_header_on_off))
    else:
        on_off_uses = use_look_df.loc[use_look_df[col_header_on_off].notnull()].copy()
        if len (on_off_uses)> 0:
            direct_result = [v for v in in_df.columns.values.tolist() if v.endswith("_0")]
            if len (on_off_uses) >0:
                on_off_uses_cols = list(set(on_off_uses['Chem Table FinalColHeader'].values.tolist()))
                print (on_off_uses_cols)
                on_off_spe_df = on_off.loc[(on_off[col_header_on_off] == 'OFF')].copy()
                spe_list = on_off_spe_df['EntityID'].values.tolist()
                for z in direct_result :
                    for p in on_off_uses_cols:
                        if z.startswith(p):
                            onoff_cols.append(z)

            for u in onoff_cols:
                in_df.loc[(in_df['EntityID'].isin(spe_list)),u]=0
        else:
            print ("   There are no on/off uses for this chemical in on/off categories: {0}".format(col_header_on_off))
    return in_df

# date
today = datetime.datetime.today()
date = today.strftime('%Y%m%d')

# Loads use look-up
use_lookup = pd.read_csv(look_up_use, low_memory=False,encoding ='ISO-8859-1')

# Loads inputs if errors occur confirm sheet names in the excel workbook and header position
try:
    # Load chemical crosswalk from the Chemical workbook into a dataframe
    if python_version_three:
        on_off_df = pd.read_excel(on_off_excel, sheet_name=sheet_name_on_off, header=0, engine='openpyxl')
    else:
        on_off_df = pd.read_excel(on_off_excel, sheetname=sheet_name_on_off, header=0, encoding="utf-8")

except Exception as error:
    # if it can't access the Label_SUUM_CoA_GIS table it will print this message and exit the script
    print ("Could not load the on/off excel workbook, please confirm the sheet name on the paths.py")
    print(error.args[0])
    sys.exit(())

on_off_df['EntityID'] = on_off_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
on_off_headers= [c for c in on_off_df if c.startswith('On')]
# Generates on/off table for Range. This is not applied to the critical habitat
all_csv= [v for v in os.listdir(final_path) if v.endswith('.csv') and not v.endswith("_OnOff.csv")]
# includes only the overlap tables from the current or most recent date
dates = [v.split("_")[3].replace('.csv','') for v in all_csv]

# confirmed the most recent table is the same as today dates, if not set date to most recent date, allowing for the on/
# off adjustment to be updated with the update of the overlap

if max(dates) != date:
    date = max(dates)

# only includes the table from the most recent date
[all_csv.remove(v) for v in all_csv if (v.split("_")[3]).replace('.csv','') != date]
list_csv = []
for table in all_csv:
    chk_date = table.split("_")[3].replace('.csv','')
    if str(chk_date) == str(date):
        list_csv.append(table)
no_on_off =[]

for table in list_csv:
    if os.path.exists(final_path + os.sep + table.replace('.csv', '_OnOff.csv')):
        os.remove(final_path + os.sep + table.replace('.csv', '_OnOff.csv'))
    out_table =final_path + os.sep + table.replace('.csv', '_OnOff.csv')
    if not os.path.exists(out_table):
        use_df = pd.read_csv (final_path + os.sep + table,low_memory=False,encoding ='ISO-8859-1')
        use_df['EntityID'] = use_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
        for header in on_off_headers:
            print ("\nWorking on the uses for {0}". format(header))
            use_df = apply_on_off(use_lookup,  on_off_df, use_df,header)
        use_df.to_csv(final_path + os.sep + table.replace('.csv', '_OnOff.csv'),encoding ='ISO-8859-1')
        print ('\nExported {0}'.format(final_path + os.sep + table.replace('.csv', '_OnOff.csv')))
    else:
        print ('Previously created {0}. Delete the file if it should be updated'.format(final_path + os.sep + table.replace('.csv', '_OnOff.csv')))

end = datetime.datetime.now()
print ("End Time: " + end.ctime())
elapsed = end - start_time
print ("Elapsed  Time: " + str(elapsed))
